/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * AgentEvent.java
 ***************************************************************************************/

package agent.base;

import java.io.Serializable;

import java.util.EventObject;

/**
 * Generic event to be passed around by agent components
 * <P>
 * Its expected that the event structure will be somewhat of a two-way
 * structure.  Recipients are permitted to modify the data (if any)
 * that the event contains, so senders should not assume that data in
 * equals data out.  It is also possible for an event recipient to
 * cancel an event distribution, by halting it.  The sender is 
 * responsible for noticing this.
 */
public class AgentEvent extends EventObject implements Serializable {
  /**
   * Status types
   */
  public static final int ACTIVE = 0;
  public static final int HALTED = 1;

  private int id;
  private int time = -1;
  private int status = ACTIVE;

  /**
   * Default constructor
   * @param source The source of the event
   * @param id The id number of the event
   */
  public AgentEvent(Object source, int id) {
    super(source);
    this.id = id;

    // Somewhat of a hack below
    try {
      agent.simplest.State state = (agent.simplest.State)agent.simplest.State.findComponent("State");
      if (state != null) {
	if (state.hasProperty("Time"))
	  time = ((Integer)state.getProperty("Time")).intValue();
      }
    } catch (Exception e) { }
    // Yes, I know I shouldn't be doing this
  }

  /**
   * Gets the event status
   */
  public int getStatus() { return status; }

  /**
   * Sets the event status
   * @param s The status number, from the statics seen above.
   */
  public synchronized void setStatus(int s) { status = s; }

  /**
   * Gets the event creation time, or -1 if not available
   */
  public int getTime() { return time; }

  /**
   * Sets the event creation time
   */
  public void setTime(int t) { time = t; }

  /**
   * Gets the event ID number
   */
  public int getID() { return id; }

  /**
   * Sets the event ID number
   */
  public void  setID(int i) { id = i ; }

  /**
   * Sets the status to halt.  Sender is responsible for noticing this.
   */
  public void halt() { setStatus(HALTED); }

  /**
   * Returns true if the event is active
   */
  public boolean isActive() { return (status == ACTIVE); }
}






