/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * AgentFrame.java
 ***************************************************************************************/

package agent.base;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.beans.Beans;
import java.util.*;
import utilities.*;
import javax.swing.JOptionPane;

/**
 */
public class AgentFrame extends Frame implements Runnable {
    private String title;
    private Vector compnames;
    private Vector comps = new Vector();
    private Dimension size;
    private boolean show = true;

    /**
     * Constructor.  Use this in conjunction with run, or use the static
     * method startApplet.
     * <p>
     * This constructor checks the system property ShowAgentFrame.  If
     * this is set to "false", then the frame will be created, but it
     * will not be shown.
     * @param t The title of the applet frame
     * @param c A list of the component names (Strings) to instantiate
     */
    public AgentFrame(String t, Vector c) {
        title = t;
        compnames = c;

        // Check this
        if (System.getProperty("ShowAgentFrame") != null) {
            if (System.getProperty("ShowAgentFrame").equalsIgnoreCase("false"))
                show = false;
        }
    }

    /**
     * Starts up the applet
     */
    public void run() {
        setTitle(title);
        makeComponents();
        if (show) {
            pack();
            setVisible(true);
            pack();
        }
    }

    /**
     * Starts up the applet
     * @param className The name of the applet class to start
     * @param title The title of the applet frame
     * @param args The arguements to pass to the applet
     */
    public void makeComponents() {
        boolean show = true;
        Panel p, q;
        ClassLoader loader = this.getClass().getClassLoader();

        setLayout(new BorderLayout());
        setBackground(Color.lightGray);

        p = new Panel();
        p.setBackground(Color.darkGray);
        p.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));

        Label l = new Label("Java Agent Framework");
        l.setFont(new Font("SansSerif", Font.BOLD, 10));
        l.setForeground(Color.lightGray);
        p.add(l);

        l.addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
                ((Component)e.getSource()).setForeground(Color.lightGray);
                JOptionPane.showMessageDialog(null,
                                              "The Java Agent Framework\n\nCore architecture written by Bryan Horling,\nwith additional help from Regis Vincent.  Other\ncomponents were developed by their authors.\n\nCopyright 1997-2002\nMulti-Agent Systems Lab\nUniversity of Massachusetts\nhttp://mas.cs.umass.edu/",
                                              "The Java Agent Framework",
                                              JOptionPane.INFORMATION_MESSAGE);
            }
            public void mouseEntered(MouseEvent e) {
                ((Component)e.getSource()).setForeground(Color.cyan);
            }
            public void mouseExited(MouseEvent e) {
                ((Component)e.getSource()).setForeground(Color.lightGray);
            }
            public void mousePressed(MouseEvent e) {}
            public void mouseReleased(MouseEvent e) {}
        });

        add(p, BorderLayout.SOUTH);

        p = new Panel();
        p.setLayout(new VFlowLayout(VFlowLayout.MIDDLE, 2, 3, true));

        Enumeration e = compnames.elements();
        while (e.hasMoreElements()) {
            String name = (String)e.nextElement();

            try {
                Log.getDefault().log("Instantiating " + name, Log.LOG_INFO);
                Object o = Beans.instantiate(loader, name);
                if (o instanceof Component) {
                    Component c = (Component)o;
                    comps.add(c);
                    p.add(c);

                } else {
                    Log.getDefault().log("Error: Object " + name + " is not a component", Log.LOG_ERR);
                }
            } catch (java.io.IOException ex) {
                Log.getDefault().log("Error: Exception loading component " + name + "\n" + ex, Log.LOG_ERR);
            } catch (ClassNotFoundException ex) {
                Log.getDefault().log("Error: Could not find component " + name + "\n" + ex, Log.LOG_ERR);
            }
        }

        add(p);
    }
}


