/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * AppletFrame.java
 ***************************************************************************************/

package agent.base;

import java.awt.*;
import java.applet.Applet;

/**
 * Applet to application frame window.  Allows you to run an applet
 * as an application, by instantiating a Applet class inside a 
 * standard Frame.
 * <P>
 * Much of this code was borrowed from the CodeWarrior Java
 * App/Applet template project.
 */
public class AppletFrame extends Frame implements Runnable {
  private String className, title, args[];

  /**
   * Constructor.  Use this in conjunction with run, or use the static
   * method startApplet.
   * @param c The name of the applet class to start
   * @param t The title of the applet frame
   * @param a The arguements to pass to the applet
   * @see #run
   * @see #startApplet
   */
  public AppletFrame(String c, String t, String a[]) {
    className = c;
    title = t;
    args = a;
  }

  /**
   * Constructor needed to pass window title to class Frame.  Used
   * by startApplet.
   * @param name The name of the enclosing frame
   */
  public AppletFrame(String name) {
    // call java.awt.Frame(String) constructor
    super(name);
  }

  /**
   * Starts up the applet
   */
  public void run() {
    startApplet(className, title, args);
  }

  /**
   * Starts up the applet
   * @param className The name of the applet class to start
   * @param title The title of the applet frame
   * @param args The arguements to pass to the applet
   */
  public static void startApplet(String className, String title, String args[]) {
    Applet a;
    Dimension appletSize;
    boolean show = true;

    // Check this
    if (System.getProperty("APPLETFRAME") != null) {
        if (System.getProperty("APPLETFRAME").equalsIgnoreCase("false"))
            show = false;
    }
    
    //System.err.println("AppletFrame startApplet starting");

    try {
      // create an instance of your applet class
      a = (Applet) Class.forName(className).newInstance();
    } catch (ClassNotFoundException e) {
      System.err.println(className + ": " + e);
      return;
    } catch (InstantiationException e) {
      System.err.println(className + ": " + e);
      return;
    } catch (IllegalAccessException e) {
      System.err.println(className + ": " + e);
      return;
    }
    
    // initialize the applet
    a.init();
    a.start();

    // create new application frame window
    AppletFrame f = new AppletFrame(title);
    
    if (show) {
        // add applet to frame window
        f.add("Center", a);
    
        // resize frame window to fit applet
        // assumes that the applet sets its own size
        // otherwise, you should set a specific size here.
        appletSize =  a.getSize();
        if ((appletSize.height < 10) || (appletSize.width < 10))
            appletSize = a.getPreferredSize();
        f.pack();
        f.setSize(appletSize);
    
        // show the window
        f.setVisible(true);
    }
    
    //System.err.println("AppletFrame startApplet done");
  }
    
  /**
   * Needed to allow window close
   */
  protected void processEvent(AWTEvent e) {

    // Window Destroy event
    if (e.getID() == Event.WINDOW_DESTROY) {
      dispose();
    }
    
    // it's good form to let the super class look at any unhandled events
    super.processEvent(e);
    
  } 
}


