/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * EventRouter.java
 *
 * #manifest: Name: agent/base/EventRouter.class
 * #manifest: Java-Bean: True
 ***************************************************************************************/

package agent.base;

import java.util.Enumeration;

/**
 * This component should serve as a reflector for agent events that you
 * want to be ordered.  So when using this, you'd first attach a normal
 * event stream to the fireEvent method.  Then attach one of the 10
 * listenerX methods (see the OrderedEventListener class) to the various
 * other components who you want to receive the original event.  Then
 * when an event is fired from the original source, it will be reflected
 * through the router, in order, to the different components which are
 * listening to the router.  In this way you can ensure that one compoent
 * will recieve an event before another, which can be useful you know.
 * <P>
 * Note this hasn't really been tested yet, so use at your own risk.
 */
public class EventRouter extends AgentComponent {
  protected ListenerVector listeners = new ListenerVector();

  /**
   * Default constructor
   */
  public EventRouter() {  }

  /**
   * Adds a listener to the list
   */
  public void addOrderedEventListener(OrderedEventListener l) {
    listeners.add(l);
  }

  /**
   * Removes a listener
   */
  public void removeOrderedEventListener(OrderedEventListener l) {
    listeners.remove(l);
  }

  /**
   * Sends an event, in order, to each of the registered listeners
   * @param e The event to send
   */
  public void fireEvent(AgentEvent e) {
    OrderedEventListener l;
    Enumeration enumr = listeners.elements();

    while (enumr.hasMoreElements()) {
      l = (OrderedEventListener)enumr.nextElement();

      if (!e.isActive()) break;

      l.listener0(e);
      l.listener1(e);
      l.listener2(e);
      l.listener3(e);
      l.listener4(e);
      l.listener5(e);
      l.listener6(e);
      l.listener7(e);
      l.listener8(e);
      l.listener9(e);
    }
  }
}
























