/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * StringArray.java
 *
 * manifest: Name: agent/base/StringArray.class
 * manifest: Java-Bean: False
 ***************************************************************************************/

package agent.base;

import java.io.Serializable;
import java.util.StringTokenizer;

/**
 * Class for containing an array of strings.  Just made this so we 
 * could have a customized property editor.
 */
public class StringArray implements Serializable {
  private String[] array;

  /**
   * Default constructor
   * @param a The array of strings to init it with
   */
  public StringArray(String[] a) {
    array = a;
  }

  /**
   * Default constructor
   * @param i The size of the array
   */
  public StringArray(int i) {
    array = new String[i];
  }

  /**
   * Set a slot
   * @param i The slot index
   * @return The string at that index
   */
  public void setSlot(int i, String s) {
    array[i] = s;
  }

  /**
   * Get a slot
   * @param i The slot index
   * @return The string at that index
   */
  public String getSlot(int i) {
    return array[i];
  }

  /**
   * The value
   * @return The true array of strings contained by the class
   */
  public String[] value() {
    return array;
  }

  /**
   * Array length
   * @return The length of the array
   */
  public int length() {
    return array.length;
  }

  /**
   * Equality tester
   * @param a The array to compare to
   */
  public boolean equals(StringArray a) {

    if (a == null)
      return false;

    if (length() != a.length())
      return false;

    else
      for (int i = 0; i < length(); i++) {
	if (getSlot(i) == null) {
	  if (a.getSlot(i) != null)
	    return false;

	} else if (! getSlot(i).equals(a.getSlot(i)))
	  return false;
      }

    return true;
  }

  /**
   * Stringify
   * @return A stringified version of the array
   */
  public String toString() {
    String s = "";

    for (int i = 0; i < array.length; i++) {
       s += array[i];
       if (i != array.length-1)
	 s += ",";
    }

    return s;
  }

  /**
   * Parses a comma deliminated string into a StringArray
   * @param s The string
   * @return A new StringArray
   */
  public static StringArray parseString(String s) {
    StringTokenizer tok = new StringTokenizer(s, ",");
    String []array = new String[tok.countTokens()];

    for (int i = 0; tok.hasMoreElements(); i++) {
      array[i] = tok.nextToken();
    }

    return new StringArray(array);
  }
}


