/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * ConflictListener.java
 ***************************************************************************************/

package agent.mass;

import agent.base.AgentListener;
import java.util.Vector;

/**
 * Interface for listening to conflict events
 */
public interface ConflictListener extends AgentListener {

  /**
   * Called when a conflict needs to be resolved.
   * This should return a vector of ConflictRecords, which
   * describe different ways the conflict could be resolved.
   * <p>
   * Do not modify the contents of the original conflict
   * record.
   */
  public Vector conflictRequest(ConflictRecord cr);

  /**
   * Called when a conflict response is received.
   * This should contain a ConflictRecord response to
   * an earlier conflict request.
   */
  public void conflictResponse(ConflictRecord cr);

  /**
   * Gets the priority of the resolver.  You should
   * override this to return your resolver's priority,
   * which should be a number between 0 and 10, with
   * 10 being the highest priority.
   */
  public int getResolverPriority();
}
