/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

package agent.mass;

import java.io.*;
import utilities.UTimer;
import utilities.Converter;
import utilities.Log;

public class DTCNative
{
    static boolean using_native = false;
    static boolean library_loaded = false;
    static boolean silent = false;

    public DTCNative()
    {
        if (!library_loaded)
        {
            boolean useit = false;
            if (System.getProperty(Scheduler.USEDTCNATIVE) != null) {
                Boolean b = (Boolean)Converter.reTypeProperty(System.getProperty(Scheduler.USEDTCNATIVE), "Boolean");
                useit = b.booleanValue();
            }
            if (useit)
            {
                String libname = System.getProperty(Scheduler.DTCLIBNAME);
                try {
                    if (libname == null)
                        libname = "/usr/maslocal/lib/libdtc-java.so";
                    System.load(libname);
                    library_loaded = true;
                    using_native = true;
                    if (!silent)
                    {
                        log("Native DTC library succesfully " +
                            "loaded.", Log.LOG_INFO);
                    }
                } catch (Throwable e) {
                    try {
                        if (libname == null)
                            libname = "dtc-java-linux";
                        log("Loading DTC library failed, " +
                            "trying again with " + libname, Log.LOG_WARNING);
                        System.loadLibrary(libname);
                        library_loaded = true;
                        using_native = true;
                        if (!silent)
                        {
                            log("Native DTC library succesfully " +
                                "loaded.", Log.LOG_INFO);
                        }
                    } catch (Throwable f) {
                        
                        if (!silent)
                        {
                            log("loading DTC library failed, " +
                                "falling back on exec call...", Log.LOG_WARNING);
                        }
                        library_loaded = false;
                        using_native = false;
                    }
                }
            } else {
                if (!silent)
                {
                    log("Not using native DTC library calls.", Log.LOG_INFO);
                }
                library_loaded = false;
                using_native = false;
            }
        }
    }

    /**
     * Logging convenience function
     */
    public void log(String s, int l) {
        //System.err.println(s);
        Log.getDefault().log(s, l);
    }

    public static boolean isNativeAvailable() { return using_native; }

    /**
     * This just calls DTC normally.  You still need to write out the taems
     * file, and read in the output file.
     * @param args the program arguments to dtc main(), same as if program was
     * being executed seperately.
     */
    public native void call_dtc_normal(String[] args);

    /**
     * Native call to custom DTC client.  This short circuits all IO, and
     * passes and returns TTaems strings.  See the dtc documentation/source for
     * more complete documentation of these arguments, or just copy someone
     * else's existing call, or use the simple call below.
     * @param numschedules the number of schedules to return.
     * @param pruning_setting a one character setting.
     * @param ttaems the ttaems string to pass in.
     * @param write_debug_file if set, writes the scheduler output to the
     * normal file.
     */
    public native String call_dtc(int numschedules, String pruning_setting, 
            int intermediate_alt_size, int tg_alt_size, int display_via_dot,
            String ttaems, boolean write_debug_file);

    /**
     * Simpler native DTC direct call.  Calls dtc with defaults taken from the
     * Scheduler.java exec call.
     */
    public String call_dtc(int numschedules, String ttaems)
    {
        return call_dtc(numschedules, "p", 50, 100, 0, ttaems, false);
    }

    static DTCNative nat;
    static private String[] temp_args;
    static private String temp_ttaems;
    static int count = 200;
    static boolean printed = false;
    public static void main(String[] args)
    {
        System.setProperty(Scheduler.USEDTCNATIVE, "true");
        System.setProperty(Scheduler.DTCLIBNAME, "libdtc-java");
        silent = false;
        nat = new DTCNative();
        temp_args = args;
        try {
            FileReader reader = new FileReader(args[0]);
            BufferedReader br = new BufferedReader(reader);
            StringBuffer buf = new StringBuffer();
            while (br.ready())
            {
                buf.append(br.readLine());
                buf.append("\n");
            }
            String ttaems = buf.toString();
            temp_ttaems = ttaems;
            printed = false;
            UTimer.time(count, new UTimer("dtc complex native call") {
                    public void action()
                    {
                        String result;
                        result = nat.call_dtc(15, "p", 50, 100, 0, temp_ttaems,
                            false);
                    } }, true);
        } catch (IOException e) {
            e.printStackTrace();
        }
        /*
        UTimer.time(count, new UTimer("dtc simple native call") {
                public void action()
                {
                    nat.call_dtc_normal(temp_args);
                } }, true);
                */
        UTimer.time(count, new UTimer("dtc exec call") {
                public void action()
                {
                    Process sched;
                    String cmd = "/nfs/ymir/u0/rawlins/bin/dtc-test ";
                    for (int i = 0; i < temp_args.length; i++)
                    {
                        cmd = cmd + temp_args[i] + " ";
                    }
                    try {
                        sched = Runtime.getRuntime().exec(cmd);
                        sched.waitFor();
                        /*
                        while(true)
                        {
                            try {
                                sched.exitValue();
                                break;
                            } catch (IllegalThreadStateException e) {
                                Thread.currentThread().sleep(100);
                            }
                        }
                        */
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } }, true);
    }
}
