/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * ProblemSolver.java
 *
 * manifest: Name: agent/mass/ProblemSolver.class
 * manifest: Java-Bean: True
 ***************************************************************************************/

package agent.mass;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.net.*;

import taems.*;
import agent.base.*;
import agent.simplest.*;

/**
 * The problem solving component.  In this case, it just produces a 
 * schedule from the conditional structure whenever a new structure
 * arrives at the State component.
 */
public class ProblemSolver extends AgentComponent implements MessageEventListener, PropertyEventListener, ActionEventListener {
  static final long serialVersionUID = 1234567890L; // kludge static ser_id.  Bad bad Bryan!
  protected Communicate communicate;
  protected State state;
  protected Log log;
  protected Execute execute;
  protected int FAC_PS;
  private Frame display;

  /**
   * Constructor
   */
  public ProblemSolver() {
    addDependency("Log");
    addDependency("State");
    addDependency("Communicate");
    addDependency("Execute");

    State.addParameterInfo("MassHome", "String", "Pathname for the Mass home directory");
  }

  /**
   * Sets up component helpers
   */
  public void init() {
    log = (Log)State.findComponent("Log");
    state = (State)State.findComponent("State");
    communicate = (Communicate)State.findComponent("Communicate");
    execute = (Execute)State.findComponent("Execute");

    FAC_PS = log.getFacilityID("ProblemSolver");

    state.addPropertyEventListener(this);
    communicate.addMessageEventListener(this);
    execute.addActionEventListener(this);
  }

  /**
   * Does stuff (well, not really...)
   */
  public void begin() { }

  /**
   * Watch for messages
   * @param m The message event to look at
   */
  public void messageSent(MessageEvent e) { }
  public void messageReceived(MessageEvent e) { }

  /**
   * Watch for interesting property changes.  Monitors Task changes.
   * @param e The event to look at
   */
  public void propertyChanged(PropertyEvent e) { 
    String key = e.getKey().toString();

    // Automatically (re)schedule if we are a simpleton
    if (key.equalsIgnoreCase("ConTaemsTask") &&
	this.getClass().getSuperclass().getName().equalsIgnoreCase("agent.base.AgentComponent")) {
      Taems taems = (Taems)e.getProperty();
      if ((taems.getFirstSchedule() == null) || (taems.getFirstSchedule().size() == 0)) {
          log.log("New task structure detected, generating schedule", Log.LOG_INFO, FAC_PS);
          Scheduler scheduler = (Scheduler)State.findComponent("Scheduler");
          if (scheduler != null) {
              Vector v = scheduler.scheduleTaems(taems);
              log.log("Got " + v.size() + " schedules, selecting the first one", Log.LOG_ERR, FAC_PS);
              taems.setScheduleVector(v);
              log.log("Schedule: " + taems.getFirstSchedule(), Log.LOG_DEBUG, FAC_PS);
              state.setProperty("ConTaemsTask", taems);
          } else
              log.log("Error, scheduler not found", Log.LOG_ERR, FAC_PS);
      }
    }
  }
  public void propertyRemoved(PropertyEvent e) { }
  public void propertyAdded(PropertyEvent e) { propertyChanged(e); }

  /**
   * Watch for actions
   * @param e The action event to look at
   */
  public void actionStarted(ActionEvent e) { }
  public void actionAborted(ActionEvent e) { }
  public void actionCompleted(ActionEvent e) {

    // Automatically quit if we are a simpleton
    if (this.getClass().getSuperclass().getName().equalsIgnoreCase("agent.base.AgentComponent")) {
      Taems conditional = (Taems)state.getProperty("ConTaemsTask");
      if (conditional.getFirstSchedule() != null) {
          if (conditional.getFirstSchedule().isCompleted() && (!execute.isExecuting())){
              ((Control)State.findComponent("Control")).quit();
          }
      }
    }
  }
}
