/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/****************************************************************
 * RandomScheduler.java
 *
 * manifest: Name: agent/mass/RandomScheduler.class
 * manifest: Java-Bean: True
 ****************************************************************/

package agent.mass;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import utilities.Distribution;
import agent.base.*;
import agent.simplest.*;
import taems.*;
import taems.parser.*;

public class RandomScheduler extends Scheduler {
    protected Random rand;

    public RandomScheduler() {
	super();
	rand = state.getRandom();
    }
    
    public void init() {
	super.init();
    }
    public void reset() {
	super.init();
    }
    
    public void end() {
	super.end();
    }

    public Vector scheduleTaems(Taems taems, boolean rescheduling, 
				   boolean set) {
       	log.log("Scheduling... ", Log.LOG_INFO, FAC_SCHED);
	Vector tmpvec=new Vector(),schedule=new Vector();
	Vector schedVec = new Vector();
	if (taems != null) {
	    // Run the scheduler for a first pass at a schedule
	    taems.setScheduleVector(null);
	    int time = 0;
	    for(Enumeration e = taems.findNodes(new Method()); 
		e.hasMoreElements(); ) {
		tmpvec.addElement((Method)e.nextElement());
	    }
	    // draw randomly from this vector
	    int sizemax= tmpvec.size();
	    for (int i=0; i< sizemax; i++){
		int pos=0;
		if (tmpvec.size() > 1)
		    pos=Math.abs(rand.nextInt()) % tmpvec.size();
		Method tmpelem=(Method)tmpvec.elementAt(pos);
		Outcome outcome =  tmpelem.getGlobalOutcome();
		tmpvec.removeElement(tmpelem);
		ScheduleElement schElement = new ScheduleElement(tmpelem, 
								 new Distribution(new Float(time), new Float(1.0)),
								 new Distribution(new Float(time+outcome.getDuration().calculateAvg()), new Float(1.0)),
								 outcome.getQuality(),
								 outcome.getDuration(),
								 outcome.getCost(), null, null);
		time = time + (int)outcome.getDuration().calculateAvg();
		schedVec.addElement(schElement);
		schedule.addElement(schElement.getLabel());
	    }
	    taems.Schedule s = new Schedule();
	    s.setScheduleElements(schedVec);
	    Vector sV = new Vector();
	    sV.addElement(s);
	    if (set)
		taems.setScheduleVector(sV);
	    log.log("Done Scheduling ", Log.LOG_INFO, FAC_SCHED);
	    return(sV);	
	}
       	log.log("The TAEMS structure was empty ! ", Log.LOG_ERR, FAC_SCHED);
	return(null);
    }

    public void propertyChanged(PropertyEvent e) { 
	String key = e.getKey().toString();
	if (key.equalsIgnoreCase("RandomSeed")) {
	    rand = state.getRandom();
	    log.log("New Random Seed = "+rand, Log.LOG_SPAM, FAC_SCHED);
	}
	else {
	    super.propertyChanged(e);
	}
    }
    public void propertyAdded(PropertyEvent e) { propertyChanged(e); }
}
