/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * RealTaemsAction.java
 ***************************************************************************************/

package agent.mass;

import java.io.*;
import java.util.*;
import java.awt.*;

import agent.simplest.*;
import taems.*;

/**
 * This is the base class for all "Real" actions which will take place
 * locally.  Override the necessary functions to produce whatever
 * behavior you want.  If you need local data, override the constructor
 * to add it initially, or use setData to add it later.  Make sure
 * you always call the constructor in this class, tho.
 */
public class RealTaemsAction extends TaemsAction implements MessageEventListener {
  Communicate c;

  /**
   * Generic constructor.  Make sure this gets called!
   */
  public RealTaemsAction(Method m, Communicate c) {
    super(m);

    setData("Local", Boolean.TRUE);
    c.addMessageEventListener(this);
  }

  /**
   * Removes us from the message event list before completing
   */
  public void complete() {
    c.removeMessageEventListener(this);
    super.complete();
  }

  /**
   * Call this method from your derived class when you are done
   */
  public void finish() {
    done = true;
  }

  /**
   * Called when a message is sent to a remote host
   */
  public void messageSent(MessageEvent e) {
  }

  /**
   * Called when a message is received from a remote host.  I assume that
   * you will use this, or some other event reception, as your primary
   * method of driving execution.
   */
  public void messageReceived(MessageEvent e) {
  }
}
