/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * ScheduleEvent.java
 ***************************************************************************************/

package agent.mass;

import java.io.*;
import java.util.*;
import taems.Schedule;

import agent.base.*;

/**
 * Generic schedule event
 */
public class ScheduleEvent extends AgentEvent {
  public static final int ADDED = 0;
  public static final int EXCEPTION = 1;
  private Schedule schedule;
  private Object source;
  private Exception ex;

  /** 
   * Constructor
   * @param source The source of the event
   * @param s The schedule in question
   * @param o The source of the schedule
   * @param id The id of the event
   */
  public ScheduleEvent(Object source, Schedule s, Object o, int id) {
    super(source, id);
    schedule = s;
    source = o;
  }

  /** 
   * Constructor
   * @param source The source of the event
   * @param s The schedule in question
   * @param o The source of the schedule
   * @param e The exception which occurred
   * @param id The id of the event
   */
  public ScheduleEvent(Object source, Schedule s, Object o, Exception e, int id) {
    super(source, id);
    schedule = s;
    source = o;
    ex = e;
  }

  /**
   * Get/set accessors
   */
  public Schedule getSchedule() { return schedule; }
  public void setSchedule(Schedule s) { schedule = s; }
  public Object getScheduleSource() { return source; }
  public void setScheduleSource(Object s) { source = s; }
  public Exception getScheduleException() { return ex; }
  public void setScheduleException(Exception e) { ex = e; }

  public String toString() {
    return "(ScheduleEvent) " + ex + "\nSchedule: " + schedule;
  }
}
