/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * FileLog.java
 *
 * manifest: Name: agent/simplest/FileLog.class
 * manifest: Depends-On: agent/simplest/Log.class
 * manifest: Java-Bean: True
 ***************************************************************************************/

package agent.simplest;

import java.io.*;
import java.awt.*;

import agent.base.*;

/**
 * Same as the base Log class, except it logs to a file.  Log messages
 * sent before the file has been initialized are handled as in the
 * base class, but with a tag attached to the message indicating
 * the problem.
 */
public class FileLog extends Log {
  static final long serialVersionUID = 1234567890L; // kludge static ser_id.  Bad bad Bryan!
  private String filename = "logfile-agent";
  private boolean append = false;

  /**
   * Constructor
   */
  public FileLog() { 
    super();
  }

  /**
   * Finalizer.  Flushes and closes the file stream.
   */
  protected void finalize() {
    closeFile();
  }

  /**
   * Filename accessor functions
   */
  public String getFileName() { return filename; }
  public void setFileName(String f) { filename = f; }

  /**
   * Append flag accessor functions.  If append is true an existing
   * log file is appended to rather than replaced
   */
  public boolean getAppend() { return append; }
  public void setAppend(boolean a) { append = a; }

  /**
   * De-serialization function that reopens our file
   */
  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    openFile();
  }

  /**
   * Start 'er up.  Opens the file.
   */
  public void init() {
    super.init();
    openFile();
  }

  /**
   * Reset.  Closes and re-opens the file.
   */
  public void reset() {
    closeFile();
    openFile();
  }

  /**
   * Opens the logfile for writing, using the correct append setting
   * @see #setFileName
   * @see #setAppend
   */
  public void openFile() {
    
    try {
      setLogStream(new FileOutputStream(filename, append));
    } catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * Closes the logfile
   */
  public void closeFile() {

    closeLogStream();
    log("Log file " + filename + " closed", 3, FAC_LOG);
  }

  /**
   * Logs a log item to a file if the file output has been initialized
   */
  protected void log(LogItem i) {

    if (output == null)
      i.setItem(i.getItem() + " #Log file not initialized");
    
    super.log(i);
  }
}























