/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * HistogramComponent.java $Revision: 1.7 $
 *
 * manifest: Name: agent/simplest/HistogramComponent.class
 * manifest: Java-Bean: False
 ***************************************************************************************/

package agent.simplest;

/* Global includes */
import java.awt.*;               // ScrollPane, PopupMenu, MenuShortcut, etc.
import java.io.*;                // Object serialization streams.
import java.util.Vector;         // To store the scribble in.

/* Local includes */
import agent.base.*;

/**
 * Histogram class
 * <P>
 * Yasmina CHITTI' Smart Contour.... Comp. Vis. Lab. UMASS  May 19, 1998<BR>
 * Modified by Regis VINCENT<BR>
 * Modified by Bryan HORLING<BR>
 */
public class HistogramComponent extends java.awt.Panel {
  protected int        data[];        // Image which has its histogram computed
  protected String     labels[];      // Xlabels
  protected Graphics   graphics;      // Graphical panel in use
  protected Vector     lines;         // Store the coordinates of each line of the histogram
  protected String     zero_label;    // Label the axis. Could add Title label
  protected String     x_label;       // if needed
  protected String     y_label;
  protected int        width,height;  // Size of the graphical panel
  protected int        histo_width;   // Width and height of the drawned 
  protected int        histo_height;  // histogram. Could be passed as arguments 
  protected Image      offscreen;     // Offscreen drawing area

  // PUBLIC CONSTRUCTORS

  /**
   * Constructor, sets empty histogram to default size
   */
  public HistogramComponent() {
    this(null, 300, 100);
  }

  /**
   * This constructor requires a desired size
   * Make the size bigger than the histogram
   * It requires an input image to compute the histogram from.
   */
  public HistogramComponent(int data[], int width, int height) {
    super();
    this.data = data;
    this.labels = null ;
    if (width != 0) { this.width = width;}
    else { this.width = 1000;} // Default value
    if (height != 0) {  this.height = height;}
    else { this.width = 700;}  // Default value
    this.histo_width = (int)Math.round(this.width * 0.8) ;    // Default value
    this.histo_height =  (int)Math.round(this.height * 0.8);  // Default value
    
    // Create a vector with a number of elements equals to 
    // the width of the histogram
    this.lines = new Vector(this.histo_width,this.histo_width); 
    // Initialise coordinates labels
    this.zero_label = new String("0");
    this.x_label = new String("X");
    this.y_label = new String("Y");
    // Create offscreen graphics space
    this.setBounds(0,0, width, height);
    makeOffscreen();
  }  

  // METHODS

  /**
   */
  public String getXLabel(int classe) {
    if ((this.labels  != null) && (classe <= this.labels.length)) 
      return(this.labels[classe]);
    else
      return(""+classe);
  }
  
  /**
   *
   */
  public int getMax(int [] data) {
      if (data.length != 0) {
	  int max=data[0];
	  for (int index = 1; index < data.length ; index++) {
	      max = Math.max(max, data[index]);
	  }
	  return(max);
      }
      return(0);
  }


    
  /** 
   * This function computes the relative lenght of all lines in the 
   * histogram fro displaying.
   * It takes nothing and returns nothing, but use a bench of
   * side effect to pass its values.
   * The Vector lines contains all the lines to display. This is
   * used by the paint function
   * @see #paint
   */
  public synchronized void  drawHisto() {
    Graphics graph;
    int   x_start_axis,y_start_axis;
    int   x_end_axis,y_end_axis;
    float tmp;
    int   index;
    int   x,y;

    makeOffscreen();

    // Compute graphical coordinates to draw the histogram
    // e.g. Compute the coordinates for drawing axis.
    x_start_axis = (int)Math.round(this.width * 0.1); // Shift x and y values from the border of
    y_end_axis   = (int)Math.round(this.height * 0.1); //the frame with an offset of 100 pixels.
    x_end_axis   = this.histo_width  + x_start_axis;
    y_start_axis = this.histo_height + y_end_axis;
    
    if (data != null) {
      int mode = getMax(data);
	  //graph = getGraphics();     // Store the graphical area
	  graph = offscreen.getGraphics();     // Store the graphical area
	  graph.setColor(Color.red); // Set the color for drawing lines
	  
	  // Compute the coordinates to draw the histogram.
	  // Scale intensity value (x) of the histogram to be ranged between
	  //  x_start_axis and x_end_axis
	  // Scale nb_of_point value (y) of the histogram to be ranged between
	  // y_start_axis and y_end_axis - Be carefull (0,0) is the upper left corner
	  // Add the new vertical line to lines vector
      // Draw the line
      if (mode != 0) {
	  for (index = 0; index < data.length ; index++) {
	      x = (int)  ( (index * this.histo_width / data.length)) + (int) x_start_axis;
	      tmp =  (float) this.histo_height * (float) data[index] / (float) mode;
	      y = (int) (y_start_axis -  tmp);
	      lines.addElement(new Line(x,y_start_axis,x,y,Color.red, ""+data[index],getXLabel(index),Color.black));
	  }
      }
    }

    // Add the axis lines in black
    lines.addElement(new Line(x_start_axis,y_start_axis,x_start_axis,y_end_axis,Color.black, "",Color.black));
    lines.addElement(new Line(x_start_axis,y_start_axis,x_end_axis,y_start_axis,Color.black,"",Color.black));
    // Set the axis labels
    this.zero_label = new String("0");
    this.x_label = new String("Observer");
    this.y_label = new String("Count");
    updateOffscreen(offscreen.getGraphics());
    repaint();
  }

  public synchronized void setHistogramData(int d[]) {
    this.data = d ;
    resetHistogramLines();
  }

  public synchronized void setHistogramLabels(String     l[]) {
    this.labels = l ;
  }

  public void resetHistogramLines() {
    this.lines.removeAllElements();
    this.lines.trimToSize();
  }

  public Dimension getPreferredSize() { return new Dimension(width, height); }

  public void setSize(Dimension d) {
    this.width = d.width;
    this.height = d.height;
    this.histo_width = (int)Math.round(this.width * 0.8) ;   
    this.histo_height =  (int)Math.round(this.height * 0.8);
    this.lines = new Vector(this.histo_width,this.histo_width); 
    this.setBounds(0,0, width, height);
    makeOffscreen(true);
    this.drawHisto();
  }

  public void makeOffscreen() { makeOffscreen(false); }
  public void makeOffscreen(boolean force) {
    // Repaint the histogram and axis
    if ((this.getSize().width != this.width) ||
	(this.getSize().height != this.height)) 
      setSize(this.getSize());

    if (force || (offscreen == null)) {
      offscreen = createImage(this.width, this.height);
      repaint();
    }
  }

  public void paint(Graphics g) {
    makeOffscreen();

    g.drawImage(offscreen, 0, 0, null);
  }

  public void update(Graphics g) {
    updateOffscreen(offscreen.getGraphics());
    paint(g);
  }
  
  public synchronized void updateOffscreen(Graphics g) {
    int index;
    if (lines.size() != 0) {
	int offset = (this.histo_width / (2 * lines.size()));
	
	g.setColor(getBackground());
	g.fillRect(0, 0, this.width, this.height);
	
	for(index = 1; index < lines.size() - 2; index++) {
	    Line l = (Line)lines.elementAt(index);
	    g.setColor(l.color);
	    g.fillRect(l.x1-offset,l.y2,  2*offset,  l.y1 - l.y2);
	    g.setColor(l.lc);
	    if (l.value != null) {
		g.setFont (new Font ("Helvetica", Font.PLAIN, 14));
		g.drawString(l.value,l.x2 - 
			     (g.getFontMetrics().stringWidth(l.value)/2),
			     l.y2 - 10);
	    }
	    
	    g.setFont (new Font ("Helvetica", Font.PLAIN, 8));
	    // Testing if we can write the label, enough room...
	    if (index < lines.size()-2) {
		if (l.label != null) {  
		    if ((g.getFontMetrics().stringWidth(l.label)) <=
			(2 * (this.histo_width / lines.size()))) {
			int y_pos;
			if ((index % 2) == 0)
			    y_pos = l.y1 + 20;
			else
			    y_pos = l.y1 + 30;
			g.drawString(l.label, l.x2 - 
			     (g.getFontMetrics().stringWidth(l.label)/2),
				     y_pos);
		    }
	  else 
	      g.drawString(""+index, l.x2 - 
			   (g.getFontMetrics().stringWidth(""+index)/2),
			   l.y1 + 20);
		}
		else
		    g.drawString(""+index, l.x2 - 
			 (g.getFontMetrics().stringWidth(""+index)/2),
				 l.y1 + 20);
	    }
	}
    
	// Repaint labels and axes
	for(index = lines.size() - 2 ; index < lines.size(); index++) {
	    Line l = (Line)lines.elementAt(index);
	    g.setColor(l.color);
	    g.drawLine(l.x1, l.y1, l.x2, l.y2);
	}
	g.setColor(Color.red);
	g.setFont (new Font ("Helvetica", Font.PLAIN, 14));
	int x_start_axis = (int)Math.round(this.width * 0.1); 
	int y_end_axis   = (int)Math.round(this.height * 0.1);
	int x_end_axis   = this.histo_width  + x_start_axis;
	int y_start_axis = this.histo_height + y_end_axis;
	g.drawString(y_label,x_start_axis - 
		     (g.getFontMetrics().stringWidth(y_label)/2),
		     y_end_axis );
	g.drawString(zero_label,x_start_axis - 20 ,y_start_axis);
	//    g.drawString(zero_label, x_start_axis,y_start_axis + 20);
	g.drawString(x_label,x_end_axis,y_start_axis);
    }
  }

  static class Line implements Serializable {
    public int x1, y1, x2, y2;
    public Color color;
    public String value,label;
    public Color lc;
    public Line(int x1, int y1, int x2, int y2, Color c, String v,
		String l, Color lc) {
      this(x1, y1, x2, y2, c, v, lc);
      this.label = l;
    }
    
    public Line(int x1, int y1, int x2, int y2, Color c, String v, 
		Color lc) {
      this.x1 = x1; this.y1 = y1; this.x2 = x2; this.y2 = y2; this.color = c;
      this.value = v;
      this.lc  = lc;
    }
  }
}
 
