/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * LogItem.java
 ***************************************************************************************/

package agent.simplest;

import java.util.Calendar;

/**
 * The object that gets logged.
 */
public class LogItem {
  public static final String ANSI_S = "\033[";
  public static final String ANSI_F = "\033[m";

  public static final String ANSI_C_RED = "31m";
  public static final String ANSI_C_GRN = "32m";
  public static final String ANSI_C_YEL = "33m";
  public static final String ANSI_C_BLU = "34m";
  public static final String ANSI_C_PUR = "35m";
  public static final String ANSI_C_CYN = "36m";
  public static final String ANSI_C_PNK = "37m";

  private String proc, thread;
  private String item;
  private int level;
  private int facility;
  private long time;
  private String aux = "";
  private Object object = null;
  private static Calendar cal = Calendar.getInstance();
  private static boolean ansi = true;

  /**
   * Constructor
   * @param o The object to log
   * @param l The log level of this item
   * @param f The log facility of this item
   */
  public LogItem(Object o, int l, int f) {
    this(o.toString(), l, f);
    object = o;
  }

  /**
   * Constructor
   * @param s The string to log
   * @param l The log level of this item
   * @param f The log facility of this item
   */
  public LogItem(String s, int l, int f) {
    this(cal.getTime().getTime(), s, l, f);
  }

  /**
   * Constructor
   * @param t The time of the logging event
   * @param o The object to log
   * @param l The log level of this item
   * @param f The log facility of this item
   */
  public LogItem(long t, Object o, int l, int f) {
    this(t, o.toString(), l, f);
    object = o;
  }

  /**
   * Constructor
   * @param t The time of the logging event
   * @param s The string to log
   * @param l The log level of this item
   * @param f The log facility of this item
   */
  public LogItem(long t, String s, int l, int f) {
    time = t;
    thread = Thread.currentThread().getName();
    item = s;
    level = l;
    facility = f;
  }

  /**
   * Clips the log item to a given length
   * @param l The clip length, or 0 if none desired
   */
  public void clip(int l) {
    
    if (l <= 0) return;
    if (item == null) return;
    if (item.length() <= l) return;

    item = item.substring(0, l) + "... [clipped]";
  }

  /**
   * Get/set the log time of this item
   */
  public long getTime() { return time; }
  public void setTime(long t) { time = t; }

  /**
   * Get/set the log level of this item
   */
  public int getLevel() { return level; }
  public void setLevel(int l) { level = l; }

  /**
   * Get/set the log facility of this item
   */
  public int getFacility() { return facility; }
  public void setFacility(int f) { facility = f; }

  /**
   * Get/set the string item to log
   */
  public String getItem() { return item; }
  public void setItem(String s) { item = s; }

  /**
   * Get/set the object (if any) being logged
   */
  public Object getObject() { return object; }
  public void setObject(Object o) { object = o; }

  /**
   * Get/set the aux data string
   */
  public String getAux() { return aux; }
  public void setAux(String a) { aux = a; }

  /**
   * Get/set the ansi flag
   */
  public static boolean getAnsi() { return ansi; }
  public static void setAnsi(boolean a) { ansi = a; }

  /**
   * Converts the item to a log-type string
   */
  public String toString() {
    //return "(" + String.valueOf(time) + "-" + thread + ") " + aux + item;
    if (ansi) {
        String aitem;
        if (level == Log.LOG_WARNING) {
          aitem = ANSI_S + ANSI_C_PUR + item + ANSI_F;
        } else if (level == Log.LOG_ERR) {
          aitem = ANSI_S + ANSI_C_RED + item + ANSI_F;
        } else {
          aitem = item;
        }
      return ANSI_S + ANSI_C_CYN + "(" + thread + ") " +  ANSI_F +
          ANSI_S + ANSI_C_GRN + "[" + aux + "] " + ANSI_F + aitem;
    } else {
        return "(" + thread + ") " + "[" + aux + "] " + item;
    }
  }
}
