/* Copyright (C) 2005, University of Massachusetts, Multi-Agent Systems Lab
 * See LICENSE for license information
 */

/***************************************************************************************
 * ScreenLog.java
 *
 * manifest: Name: agent/simplest/ScreenLog.class
 * manifest: Depends-On: agent/simplest/Log.class
 * manifest: Java-Bean: True
 ***************************************************************************************/

package agent.simplest;

import java.io.*;
import java.awt.*;
import javax.swing.*;

import agent.base.*;

/**
 * Same as the base Log class, except it logs to a file.  Log messages
 * sent before the file has been initialized are handled as in the
 * base class, but with a tag attached to the message indicating
 * the problem.
 */
public class ScreenLog extends Log {
  static final long serialVersionUID = 1234567890L; // kludge static ser_id.  Bad bad Bryan!
  private String title = "Screen Log ";
  private static int NUM = 1;
  private transient JFrame window;
  private transient TextArea text;
  private int row = 5, col = 80;
  private boolean append = true;

  /**
   * Constructor
   */
  public ScreenLog() { 
    super();

    FAC_LOG = getFacilityID("Log");

    removeAll();
  }

  /**
   * Finalizer.  Flushes and closes the file stream.
   */
  protected void finalize() {
    if (window != null)
      window.dispose();
  }

  /**
   * Start 'er up.  Makes and opens the log window.
   */
  public void init() {
    super.init();

    if (State.hasComponent("WindowManager")) {
      WindowManager wm = (WindowManager)state.findComponent("WindowManager");
      window = wm.createWindow(title + NUM++);
    } else
      window = new JFrame(title + NUM++);

    text = new TextArea("", row, col, TextArea.SCROLLBARS_BOTH);
    text.setEditable(false);
    window.getContentPane().add(text);
    window.pack();
    if (!agent.simplest.State.hasComponent("WindowManager"))
      window.setVisible(true);
  }

  /**
   * Reset.  Closes and re-opens the file.
   */
  public void reset() {
    text.setText("");
  }

  /**
   * Gets the window's title
   * @return The title
   */
  public String getWindowTitle() { return title; }

  /**
   * Sets the window's title
   * @param t The new title
   */
  public void setWindowTitle(String t) {
    title = t;
    if (window != null)
      window.setTitle(t);
  }

  /**
   * Gets the append flag
   * @return The append flag
   */
  public boolean getAppend() { return append; }

  /**
   * Sets the append flag
   * @param a The append flag
   */
  public void setAppend(boolean a) { append = a; }

  /**
   * Gets the number of rows
   * @return The number of rows
   */
  public int getRows() { return row; }

  /**
   * Sets the number of rows
   * @param r The number of rows
   */
  public void setRows(int r) {
    row = r;
    if (text != null)
      text.setRows(row);
  }

  /**
   * Gets the number of cols
   * @return The number of cols
   */
  public int getCols() { return col; }

  /**
   * Sets the number of cols
   * @param c The number of cols
   */
  public void setCols(int c) {
    col = c;
    if (text != null)
      text.setColumns(col);
  }

  /**
   * Logs a log item to a file if the file output has been initialized
   */
  protected void log(LogItem i) {

    if (text == null) {
      i.setItem(i.getItem() + " #ScreenLog not initialized");   
      super.log(i);
    } else {
      i.clip(getLogLength());
      
      if (loggable(i)) {
	i.setAux(clock + ":" + name);
	
	if (append)
	  text.append(i.toString() + "\n");
	else
	  text.insert(i.toString() + "\n", 0);
      }
    }
  }
}
